package com.jpmorrsn.javaFBP;


/** This class is based on a (language) parsing technique called Babel, that I learned in
*  England many years ago!
*  This class definitely has an internal state, so all users must create an instance of it.  
**/
 
public class BabelParser {


	
	static final String copyright = 
	"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
	"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
	"based on the Everything Development Company's Artistic License.  A document describing " +
	"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
	"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";
		
		   
		private char input[];
		private char output[];
		private int inputIndex;
		private int outputIndex;
		private int endOfInput;
		
		private InputPort inport;
		private Packet currentPacket = null;
		
	/**
	 * BabelParser constructor.
	 */
	public BabelParser(InputPort input) {
		super();
		inputIndex = 0;
		endOfInput = 0;     // this will force getMoreInput before first comparison
		outputIndex = 0;
		output = new char[256];
		inport = input;
	}
	/**
	 * Erase output stream
	 */
	public void eraseOutput() {
		
		outputIndex = 0;
		
		}
	/**
	 * Get another input Packet - return false if no more
	 */
		boolean getMoreInput() {
			if (null != currentPacket) currentPacket.drop();
		 
			if (null == (currentPacket = inport.receive())) 
			  return false;                 //   end of file returns false 
			String str = (String)currentPacket.getContent();
			int x = str.length(); 
		    input = new char[x];            // new char array each time
		    inputIndex = 0;
		    endOfInput = x;
		
		    str.getChars(0, x, input, 0);   // move str into input array
		    return true;
		}
	/**
	 * Take output generated by syntax scan, and return as String
	 * Output index is then set back to zero. 
	 * @return java.lang.String
	 */
	public String getOutStr() {
		String sym = null;
		if (outputIndex > 0) {
		    try {sym = new String(output, 0, outputIndex);}
			    catch (NullPointerException r) {
							System.out.println("RuntimeException:" + r);
	
			    }
			    catch (IndexOutOfBoundsException e) {
							System.out.println("RuntimeException:" + e);
	
			    }
		    outputIndex = 0;
		}
		return sym;
	}
	/**
	 * This macro compares against a given character. Scanning is continuous from the end of one
	 *  incoming packet to the start of the next one.  End of input results in a false result -
	 *  i.e. end of input is considered to not match _any_ test character.
	 */
	public boolean tc(char x)  {
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }
		  if (input[inputIndex] != x) return false;
		  output[outputIndex] = input[inputIndex];
		  outputIndex++;
		  inputIndex++;
		  return true;
		}
	/**
	 * Same as tc(char), but with modification (must be 'n' or 'o')
	 * ('n' is equivalent to old Babel 'IO' - I- and O-modification)
	 * @param x char
	 * @param mod char
	 */
	
		public boolean tc(char x, char mod) {
	
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }	
		  if (input[inputIndex] != x) return false;
		
		  if (mod == 'o') 
		     inputIndex++;
		  return true;
		}
	/**
	 * This macro compares against a number (figure). Scanning is continuous from the end of one
	 *  incoming packet to the start of the next one.  End of input results in a false result.
	 */
	public boolean tf()  {
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }
		  if (!Character.isDigit(input[inputIndex])) return false;
		  output[outputIndex] = input[inputIndex];
		  outputIndex++;
		  inputIndex++;
		  return true;
		}
	/**
	 * Same as tf(), but with modification (must be 'o' or 'n')
	  * ('n' is equivalent to old Babel 'IO' - I- and O-modification)
	 * @param mod char
	 */
	
		public boolean tf(char mod) {
	
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }	
		  if (!Character.isDigit(input[inputIndex])) return false;
		
		  if (mod == 'o') 
		     inputIndex++;
		  return true;
		}
	/**
	 * This macro compares against a letter. Scanning is continuous from the end of one
	 *  incoming packet to the start of the next one.  End of input results in a false result.
	 */
	public boolean tl()  {
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }
		  if (!Character.isLetter(input[inputIndex])) return false;
		  output[outputIndex] = input[inputIndex];
		  outputIndex++;
		  inputIndex++;
		  return true;
		}
	/**
	 * Same as tl(), but with modification (must be 'o' or 'n')
	  * ('n' is equivalent to old Babel 'IO' - I- and O-modification)
	 * @param mod char
	 */
	
		public boolean tl(char mod) {
	
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }	
		  if (!Character.isLetter(input[inputIndex])) return false;
		  
		  if (mod == 'o') 
		     inputIndex++;
		  return true;
		}
	/** 
	* Babel 'universal comparator' - it is always true, unless we are at end of file 
	*/
	public boolean tu()  {
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }	 
				 
		  output[outputIndex] = input[inputIndex];
		  outputIndex++;
		  inputIndex++;
		  return true;
		
		}
	/**
	 * Same as tu(), but with modification (must be 'o' or 'n')
	  * ('n' is equivalent to old Babel 'IO' - I- and O-modification)
	 * @param mod char
	 */
	public boolean tu(char mod)   {
		  while (inputIndex >= endOfInput) {
			if (!getMoreInput())
			   return false;
		    }
		
		  
		  if (mod == 'o') 
		     inputIndex++;
		  return true;  
		}
	}

