package com.jpmorrsn.javaFBP;

import java.lang.Object;


/** This class provides connections that hold just a single object
*  at network setup time.  The <code>initialize</code> statement in
*  the network configuration mini-language creates instances of
*  this class.  It is a degenerate form of Connection.
*  <p> This class implements a type of parametrization of components -
*  the "parameter", which can be any object type, is associated with a port,
*  and is turned into a Packet when the first receive to that port is issued.
*  This occurs once per activation of that component.   From the component's
*  point of view, it looks like a normal data stream containing one Packet.
*/



final class InitializationConnection  implements InputPort {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";


	Component receiver;  // The receiver to deliver to.

	Packet packet;

	Object content;  // object passed to it by initialize statement


	/** Create an InitializationConnection: requires a content and a receiver.
	*/

	InitializationConnection(Object content, Component newReceiver) {

		this.content = content;   // store object
		receiver = newReceiver;
		}
	/** The maximum number of packets available in an InitializationConnection must be 1.
	*/
	public int capacity() {
		return 1;
		}
/**
 * Close Initialization Connection (dummy method)
  */
public void close() {}
/**
 * Return 1 as number of packets in InitializationConnection.
 * @return int
 */
public int count() {
	return 1;
}
/**  Invoked to get receiver.
*/

public Component getReceiver() {  // added for subnet support 
	return receiver;
	}	
	/** The receive function of an InitializationConection.
	*  Returns null after the packet has been delivered (because the Packet is set to null).
	* You get one copy per activation
	*
	* Warning: the object contained in this packet must not be modified.
	*
	* See InputPort.receive.
	*/
	public Packet receive() {
		Packet p = packet;
		// p.setOwner(receiver);
		packet = null;
		return p;
		}
	public void setReceiver(Component newReceiver) {  // added for subnet support 
	    receiver = newReceiver;
		}
/**
 * Invoked to tell us the type of packet content being sent
 * or expected.  The receiver's type must be a supertype of
 * content, or the network is ill-formed.
 */
	public void setType(Class type) {

		if (type == null) return;

		if (type == Object.class) return;

	/*	if (!(type.isAssignableFrom(content.getClass())))
				FlowError.complain("Connection type mismatch");
    */
		}
}
