package com.jpmorrsn.javaFBP;

// import java.util.Vector;


/**
 * This class is used within components to declare instance variables
 * that hold output ports.  Such instance variables should be assigned
 * within the <code>openPorts</code> routine of the component and never
 * changed thereafter.  Packets can be sent, and the status of the
 * port manipulated, using the API specified by this interface.
 * Unlike InputPort, this is a real class, the difference being that 
 * multiple InputPorts can feed a single Connection, whereas a 
 * Connection can not "feed" more than one InputPort. 
 **/

public class OutputPort {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";

	   Connection cnxt = null;     // downstream connection
	   boolean isClosed = false;
	   Component sender;           // component sending to this Output Port

/**
 * Close this OutputPort.  This is a signal that no further packets
 * will be sent via this OutputPort.  Since more than one OutputPort may feed a given
 * Connection, this does not necessarily close the Connection.
 **/

	public void close() {
		   if (!isClosed) {
		       isClosed = true;
		       synchronized(cnxt) {
		       if (!cnxt.isClosed()) 
		           cnxt.closeSender(); // indicate that one sender has terminated
		       }
		   }
		   }
		   
		   
/**
 * This method returns the downstream Packet count for a given OutputPort
 * It is normally only used by components that do load balancing.
 * @return int
 */
public int downstreamCount() {
	return cnxt.count();
}
/**
 * Send a packet to this Port.
 * The thread is suspended if no capacity is currently available.
 * If the port or connection has been closed, <code>false</code> is
 * returned; otherwise, <code>true</code> is returned.
 * <p> Do not reference the packet after sending - someone else may be
 *   modifying it!
 * @param packet packet to send
 * @return true if successful
 **/

	// The send function.
	public boolean send(Packet packet) {
	  boolean res;

	 
	  //Thread t = Thread.currentThread();
		//if (t != packet.owner)
	  if (sender != packet.owner)
		   FlowError.complain("Packet being sent not owned by current component");

	  if (isClosed)
		  return false;
	  res = cnxt.send(packet, this);  // fire up send method on connection

	  return res;
		}
/**
 * Set sender reference
 */
public void setSender(Component c) {

	sender = c;
	}
	/** Invoked to tell us the type of packet content being sent
	 * or expected.  The receiver's type must be a supertype of
	 * every sender's type, or the network is ill-formed.
	 */


	public void setType(Class type) {

		/*
		if (type == null) return;

		if (cnxt.senderTypes == null)
		    cnxt.senderTypes = new Vector();
		cnxt.senderTypes.addElement(type);

		*/

		}
}
