package com.jpmorrsn.javaFBP.verbs;



import com.jpmorrsn.javaFBP.*;

import com.jpmorrsn.javaFBP.test.parms.*;



/** Component to collate two or more streams of packets, based 
 *   on the 1st 10 characters of each IP
*/

public class Collate extends Component {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";

	

	InputPort[] inportArray;
	OutputPort outport;
	InputPort ctlfldport;

	String prev = null;
	String hold = null;
	int low;
	Packet pArray[];
	int parmct;
	CollateParm cp = null;
	
	protected void execute() throws Throwable {
		
		Packet cfp = ctlfldport.receive();
		if (cfp == null) 
			return;
				
		cp = (CollateParm)(cfp.getContent());
		cfp.drop();
		
		// System.out.println(cp.fldArray.length);
		
		parmct = cp.fldArray.length;
		int totlen = 0;
		for (int i = 0; i < parmct; i++)
             totlen += cp.fldArray[i];
		
		for (int i = 0; i < parmct; i++) {
		Packet p2 = create(Packet.OPEN, " ");
		if (!(outport.send(p2))) 
		     p2.drop();
		}
		int no = inportArray.length;
		int count = no;
		pArray = new Packet[no];			
	    Packet p;
	    for (int i = 0; i < no; i++) { 
		    p = inportArray[i].receive();
		    if (p == null) { 
		    	pArray[i] = null;
		    	--count;
		    }
		       else pArray[i] = p;
 			
			}
	     
		 while (true) {
			 hold = "\uffff";
		     low = 0;
			
			 for (int i = 0; i < no; i++) { 
				 if (pArray[i] != null) {
					
					 String value = (String) pArray[i].getContent();
					 value = value.substring(0,totlen);
					 if (value.compareTo(hold) < 0) {
						 hold = value;
 						 low = i;
 						 }
				 }
				 
				
			 }
			 sendOutput(low);
			 pArray[low] = null;
			 p = inportArray[low].receive();
			 if (p == null)
				 count --;
		     else pArray[low] = p;
			 if (count == 0) break;
			 
		 }
		 for (int i = 0; i < parmct; i++) {
		 Packet p2 = create(Packet.CLOSE, " ");
		 if (!(outport.send(p2))) 
		     p2.drop();
		 }
	}
	void sendOutput(int x) {
		if (prev != null){
			//if (hold.compareTo(prev) != 0) {
			int level = findLevel(hold, prev);
			for (int i = 0; i < level; i++){	
				Packet p2 = create(Packet.CLOSE, " ");
				if (!(outport.send(p2))) 
				     p2.drop();
			}
			for (int i = 0; i < level; i++){		
				Packet p2 = create(Packet.OPEN, " ");
				if (!(outport.send(p2))) 
				     p2.drop();
		    }
			
		}
		if (!(outport.send(pArray[x]))) 
			pArray[x].drop();
		prev = hold;
	}
	
	int findLevel(String s, String t) {
		int j = 0;
		for (int i = 0; i < parmct; i++){
			String h1 = hold.substring(j, j + cp.fldArray[i]);
			String p1 = prev.substring(j, j + cp.fldArray[i]);
			if (h1.compareTo(p1) != 0)
                 return parmct - i;
			j += cp.fldArray[i];
	    }
		return 0;
	}	
	protected Object[] introspect() {
		return new Object[] {
		"concatenates input streams at array port IN and sends them to port OUT",
		"IN", "input array", Object.class,
			"multiple input streams",
		"CTLFIELDS", "input",Object.class,  // array of integers
			"control field array",
		"OUT", "output", Object.class,
			"output stream"};
		}
	protected void openPorts() {
		
		inportArray = openInputArray("IN");
	//		inport.setType(Object.class);
		ctlfldport = openInput("CTLFIELDS");

		outport = openOutput("OUT");
			
		}
	
}
