package com.jpmorrsn.javaFBP.verbs;

import com.jpmorrsn.javaFBP.*;



/** Component to assign incoming packets to the output port that has the
* smallest backlog of packets waiting to be processed.
*/


public class LoadBalance extends Component {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";



	InputPort inport;
	OutputPort[] outportArray;

	protected void execute() throws Throwable {

		int no = outportArray.length;
		int backlog;
		int sel = -1;

	    Packet p;
		while ((p = inport.receive()) != null) {
			backlog = Integer.MAX_VALUE;
			for (int i = 0; i < no; i++) {
			    int j = outportArray[i].downstreamCount();
			    if (j < backlog) {
				    backlog = j;
				    sel = i;
			        }
			    }
			    if (!(outportArray[sel].send(p))) {
			    	p.drop();
			    	}

			}
	}
	protected Object[] introspect() {
		return new Object[] {
		"obtains input stream at port IN and distributes them to array port OUT" +
		" depending on which output port has the smallest backlog" ,
		"IN", "input", Object.class,
			"input stream",
		"OUT", "output", Object.class,
			"multiple output streams"};
		}
	protected void openPorts() {

		inport = openInput("IN");
		inport.setType(Object.class);

		outportArray = openOutputArray("OUT");

		}
}
