package com.jpmorrsn.javaFBP.verbs;

import com.jpmorrsn.javaFBP.*;
import java.io.Reader;
import java.io.BufferedReader;



/** Component to read data from a file or other input medium, generating
* a stream of packets.   The data source is specified as a Stream Reader
* via an InitializationConnection.
*/

public class ReadFile extends Component {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";



	OutputPort outport;

	InputPort source;


	protected void execute() throws Throwable {
		Packet rp = source.receive();
		if (rp == null) {
			return;
			}
		BufferedReader b = new BufferedReader((Reader)(rp.getContent()));
		rp.drop();
		String s;
		while ((s = b.readLine()) != null) {
			Packet p = create(s);
			
			if (!(outport.send(p))) {
				p.drop();
				return;
				}
			}

		b.close();
		}
	protected Object[] introspect() {
		return new Object[] {
		"reads input from a java.io.Reader " +
		"(a character stream) and outputs it line-by-line",
		"OUT", "output", String.class,
			"lines read",
		"SOURCE", "parameter", Reader.class,
			"Reader to read lines from"};
		}
	protected void openPorts() {

		outport = openOutput("OUT");
		outport.setType(String.class);

		source = openInput("SOURCE");
		source.setType(Reader.class);

		}
}
