package com.jpmorrsn.javaFBP.verbs;

import com.jpmorrsn.javaFBP.*;



/** Component to split an input stream into multiple output streams,
* where the first 30 packets go to the first output port, the next 30 go
* to the second and so on.  Each output stream is closed before data starts
* being sent to the next.  This component was used for testing deadlock behaviour.
*/



public class Splitter1 extends Component {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";



	InputPort inport;
	OutputPort[] outportArray;

	protected void execute() throws Throwable {

		int no = outportArray.length;
		int e = 0;
		int i = 0;
		Packet p;

		while ((p = inport.receive()) != null) {

			if (!(outportArray[e].send(p))) {
			    	p.drop();
			    	}
			if (i == 30) {
				if (e < no - 1) {         // if last output stream, don't close it
				    outportArray[e].close();
				    ++e;
			    	}
				i = 0;
			    }
			else
			    i++;

			}
		while (e < no) {
			outportArray[e].close();
			e++;
	    	}

	}
	protected Object[] introspect() {
		return new Object[] {
		"splits input stream at port IN, sending 30 IPs to each element of array port OUT",
		"IN", "input", Object.class,
			"input stream",
		"OUT", "output", Object.class,
			"multiple output streams"};
		}
	protected void openPorts() {

		inport = openInput("IN");
		inport.setType(Object.class);

		outportArray = openOutputArray("OUT");

		}
}
