package com.jpmorrsn.javaFBP.verbs;

import com.jpmorrsn.javaFBP.*;

import java.io.*;


/** Component to write data to a file or other output medium, using
* a stream of packets.   The data target is specified as a Stream Writer
* via an InitializationConnection.
* It is specified as "must run" so that the output file will be cleared
* even if no data packets are input.
*/

public class WriteFile extends Component implements MustRun  {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";


	InputPort inport;

	InputPort destination;

	InputPort flush;

	static String linesep = System.getProperty("line.separator");


	protected void execute() throws Throwable {
		Packet wp = destination.receive();
		if (wp == null) 
			return;
			
		Packet fp = flush.receive();
		
		Writer w = (Writer)(wp.getContent());
		
		wp.drop();
		Packet p;
		while ((p = inport.receive()) != null) {
			try {w.write((String)(p.getContent()));}
			catch (IOException e) {System.err.println(e.getMessage()+ " - component: " + this.getName());};
			w.write(linesep);
			
			if (fp != null) w.flush();
			p.drop();
			}
		w.close();
		if (fp != null) fp.drop();
		}
	protected Object[] introspect() {
		return new Object[] {
		"transmits its input to a java.io.Writer as lines, " +
		"optionally flushing the Writer after each line",
		"IN", "input", String.class,
			"lines to be written",
		"DESTINATION", "parameter", Writer.class,
			"Writer to write lines to",
		"FLUSH", "parameter", null,
			"flush Writer after every line if present"};
		}
	protected void openPorts() {
		inport = openInput("IN");
		inport.setType(String.class);

		destination = openInput("DESTINATION");
		destination.setType(Writer.class);

		flush = openInput("FLUSH");
		}
}
