package com.jpmorrsn.javaFBP.verbs;

import com.jpmorrsn.javaFBP.*;


import java.util.Stack;
import java.util.Date;
import java.util.HashMap;


/** Component to scan off XML and generate data packets.  The generation of packets
* and setting of object attributes have been stubbed, and diagnostic messages has been
* placed under switch control, to allow performance measurements.
* The method 'createTables' has been preset to a number of classes and their attributes - 
* this should be modified to be self-generating as far as possible.
*/


public class XMLToObj extends Component {

static final String copyright = 
"Copyright 1999, 2000, 2001, 2002, J. Paul Morrison.  At your option, you may copy, " +
"distribute, or make derivative works under the terms of the Clarified Artistic License, " +
"based on the Everything Development Company's Artistic License.  A document describing " +
"this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. " +
"THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.";

	

	InputPort inport;
	OutputPort outport;
	HashMap<String,HashMap> classNames = null;
	HashMap<String,Class> classes = null;
	
 public static class Order {
	public Date expiryDate;
	public Date webOrderDate;
	}
	
/**
 * Create scan tables for component
 * In the field list hash maps, a value starting with '*' indicates an atomic data type.
 * Other values are class names.
 */
public void createTables() {
	
	HashMap<String, String> fl1 = new HashMap<String, String>();
	fl1.put("HeaderReq", "CHeaderReq");
	fl1.put("Parms", "CParms");
	fl1.put("getAccountsSummary", "*");
	
	HashMap<String, String> fl2 = new HashMap<String, String>();
	fl2.put("service", "*");
	fl2.put("channelId", "*");
	fl2.put("language", "*");
	fl2.put("getUserInfo", "*");

	HashMap<String, String> fl3 = new HashMap<String, String>();
	fl3.put("userId", "*");
	fl3.put("password", "*");

	HashMap<String, String> fl11 = new HashMap<String, String>();
	fl11.put("HeaderRes", "CHeaderRes");
	fl11.put("AccountsSummaryRes", "CAccountsSummaryRes");

	HashMap<String, String> fl12 = new HashMap<String, String>();
	fl12.put("SessionToken", "CSessionToken");
	fl12.put("service", "*");
	fl12.put("channelId", "*");
	fl12.put("language", "*");
	fl12.put("processTimeStmp", "*");
	fl12.put("Broadcast", "CBroadcast");
	fl12.put("UserInfo", "CUserInfo");

	HashMap<String, String> fl13 = new HashMap<String, String>();
	fl13.put("sessionID", "*");
	fl13.put("tracker", "*");

	HashMap<String, String> fl14 = new HashMap<String, String>();
	fl14.put("b.TimeStamp", "*");
	fl14.put("messageSubject", "*");
	fl14.put("messageText", "*");
	fl14.put("acknowledge", "*");

	HashMap<String, String> fl15 = new HashMap<String, String>();
	fl15.put("userName", "*");
	fl15.put("CulturalPref", "CCulturalPref");
	fl15.put("AccountLst", "CAccountLst");
	fl15.put("PermLst", "CPermLst");

	HashMap<String, String> fl16 = new HashMap<String, String>();
	fl16.put("LanguageLst", "CLanguageLst");
	fl16.put("NumMask", "CNumMask");
	fl16.put("MoneyMask", "CMoneyMask");
	fl16.put("dateMask", "*");
	fl16.put("timeMask", "*");
	fl16.put("timeStampSep", "*");
	
	HashMap<String, String> fl17 = new HashMap<String, String>();
	fl17.put("language", "*");
	
	HashMap<String, String> fl18 = new HashMap<String, String>();
	fl18.put("thousSep", "*");
	fl18.put("decSep", "*");
	
	HashMap<String, String> fl19 = new HashMap<String, String>();
	fl19.put("currencyBefAft", "*");
	fl19.put("currencySep", "*");
	
	HashMap<String, String> fl20 = new HashMap<String, String>();
	fl20.put("accountId", "*");
		
	HashMap<String, String> fl21 = new HashMap<String, String>();
	fl21.put("Perm", "CPerm");
		
	HashMap<String, String> fl22 = new HashMap<String, String>();
	fl22.put("resourceId", "*");
	fl22.put("permInd", "*");
	
	HashMap<String, String> fl23 = new HashMap<String, String>();
	fl23.put("as.TimeStamp", "*");
	fl23.put("AccountSummaryLst", "CAccountSummaryLst");
	fl23.put("TotalByCurrencyLst", "CTotalByCurrencyLst");
	fl23.put("ExchRateLst", "CExchRateLst");
	fl23.put("GrandTotals", "CGrandTotals");
	fl23.put("AssetDistribution", "CAssetDistribution");
	
	HashMap<String, String> fl24 = new HashMap<String, String>();
	fl24.put("AccountSummary", "CAccountSummary");
	
	HashMap<String, String> fl25 = new HashMap<String, String>();
	fl25.put("rRepId", "*");
	fl25.put("accountId", "*");
	fl25.put("accountType", "*");
	fl25.put("accountCurrency", "*");
	fl25.put("tdMoneyBal", "*");
	fl25.put("tdSecurityValue", "*");
	fl25.put("tdTotalMarketVal", "*");
	fl25.put("tdMarginBal", "*");
	fl25.put("sdMoneyBal", "*");
	fl25.put("sdSecurityValue", "*");
	fl25.put("sdTotalMarketVal", "*");
	fl25.put("sdMarginBal", "*");
	fl25.put("ServiceLst", "CServiceLst");
	
	HashMap<String, String> fl26 = new HashMap<String, String>();
	fl26.put("itemService", "*");
	
	HashMap<String,String> fl27 = new HashMap<String,String>();
	fl27.put("TotalByCurrency", "CTotalByCurrency");
	
	HashMap<String,String> fl28 = new HashMap<String,String>();
	fl28.put("tdTotalCash", "*");
	fl28.put("tdTotalSecMVal", "*");
	fl28.put("tdTotalMVal", "*");
	fl28.put("tdTotalMargin", "*");
	fl28.put("sdTotalCash", "*");
	fl28.put("sdTotalSecMVal", "*");
	fl28.put("sdTotalMVal", "*");
	fl28.put("sdTotalMargin", "*");
		
	HashMap<String,String> fl29 = new HashMap<String,String>();
	fl29.put("ExchRate", "CExchRate");
		
	HashMap<String,String> fl30 = new HashMap<String,String>();
	fl30.put("currencySource", "*");
	fl30.put("currencyTarget", "*");
	fl30.put("rate", "*");
	fl30.put("effectiveDate", "*");
		
	HashMap<String,String> fl31 = new HashMap<String,String>();
	fl31.put("tdGrandTotalCash", "*");
	fl31.put("tdGrandTotalSecVal", "*");
	fl31.put("tdGrandTotalMktVal", "*");
	fl31.put("tdGrandTotalMargin", "*");
	fl31.put("sdGrandTotalCash", "*");
	fl31.put("sdGrandTotalSecVal", "*");
	fl31.put("sdGrandTotalMktVal", "*");
	fl31.put("sdGrandTotalMargin", "*");
	
	HashMap<String,String> fl32 = new HashMap<String,String>();
	fl32.put("assetTypingId", "*");
	fl32.put("typeTag", "*");
	fl32.put("typeValue", "*");
	fl32.put("typePercent", "*");
	
	HashMap<String,String> fl33 = new HashMap<String,String>();
	fl33.put("webOrderDate", "*");
	fl33.put("expiryDate", "*");
	
	
	
	/*---------------------------------------------------------------------------*/
	
	classNames = new HashMap<String,HashMap>();
	classNames.put("LoginReq", fl1);
	classNames.put("CHeaderReq", fl2);
	classNames.put("CParms", fl3);

	classNames.put("LoginRes", fl11);
	classNames.put("CHeaderRes", fl12);
	classNames.put("CSessionToken", fl13);
	classNames.put("CBroadcast", fl14);
	classNames.put("CUserInfo", fl15);
	classNames.put("CCulturalPref", fl16);
	classNames.put("CLanguageLst", fl17);
	classNames.put("CNumMask", fl18);
	classNames.put("CMoneyMask", fl19);
	classNames.put("CAccountLst", fl20);
	classNames.put("CPermLst", fl21);
	classNames.put("CPerm", fl22);
	classNames.put("CAccountsSummaryRes", fl23);
	classNames.put("CAccountSummaryLst", fl24);
	classNames.put("CAccountSummary", fl25);
	classNames.put("CServiceLst", fl26);
	classNames.put("CTotalByCurrencyLst", fl27);
	classNames.put("CTotalByCurrency", fl28);
	classNames.put("CExchRateLst", fl29);
	classNames.put("CExchRate", fl30);
	classNames.put("CGrandTotals", fl31);
	classNames.put("Order", fl33);

	
	classes = new HashMap<String,Class>();
	classes.put("LoginReq", Date.class);
	classes.put("CHeaderReq", Date.class);
	classes.put("CParms", Date.class);
	
	classes.put("LoginRes", Date.class);
	classes.put("CHeaderRes", Date.class);
	classes.put("CSessionToken", Date.class);
	classes.put("CBroadcast", Date.class);
	classes.put("CUserInfo", Date.class);
	classes.put("CCulturalPref", Date.class);
	classes.put("CLanguageLst", Date.class);
	classes.put("CNumMask", Date.class);
	classes.put("CMoneyMask", Date.class);
	classes.put("CAccountLst", Date.class);
	classes.put("CPermLst", Date.class);
	classes.put("CPerm", Date.class);
	classes.put("CAccountsSummaryRes", Date.class);
	classes.put("CAccountSummaryLst", Date.class);
	classes.put("CAccountSummary", Date.class);
	classes.put("CServiceLst", Date.class);
	classes.put("CTotalByCurrencyLst", Date.class);
	classes.put("CTotalByCurrency", Date.class);
	classes.put("CExchRateLst", Date.class);
	classes.put("CExchRate", Date.class);
	classes.put("CGrandTotals", Date.class);
	classes.put("CAssetDistribution", Date.class);
	classes.put("Order", Date.class);











	
}
	protected void execute() throws Throwable {
	     
	BabelParser bp = new BabelParser(inport);

	createTables();

	long now = System.currentTimeMillis();
	
	boolean done = false;
	int tagCount = 0;
	
	String sym = "";
	String data = null;
	Object object = null;

	HashMap curFl = null;
	boolean atomic = false;

	// 3 stacks
	Stack<String> names = new Stack<String>();         // names (atomic and non-atomic) 
	Stack<HashMap> fldLists = new Stack<HashMap>();      // field lists 
	Stack<Object> objs = new Stack<Object>();          // class instances
	Stack<String> clsNames = new Stack<String>();      // class names 
	
	boolean endsw;  // if true, it is an end tag
	boolean control = false;  // if true, it is a comment or control tag

	String type;
	
	while (true) {                           // skip blanks or tabs
	    if (bp.tc(' ', 'o')) continue;
	    if (bp.tc('\t', 'o')) continue;
	    break;
	    }  
	    
	if (!bp.tc('<', 'o')) {
		network.GenTraceLine("1st non-blank char not '<'");
		// terminate();
		return;
	    }
	 
	while (true)  {
		tagCount++;
		
		if (bp.tc('/', 'o')) endsw = true;
		  else endsw = false;

		if (bp.tc('!', 'o')) control = true;
		if (bp.tc('?', 'o')) control = true;		  
		
		while (true) {
		
	 		if (bp.tc('>', 'o')) break;
	 		if (bp.tc(' ', 'o')) 
		 		while (true) {
		 		    if (bp.tc('>', 'n')) break;  // 'n' is equiv to 'io'
		 		    bp.tu('o');
	 		        }
		 	if (bp.tc('>', 'o')) break;	
			bp.tu();
		    }  		// at end, we can use getOutStr to get output into String (sym)


		if (!control) {
			
		    sym = bp.getOutStr();

			if (sym.charAt(sym.length() - 1) == '/')  {  // stand-alone field (no data)
				endsw = true;
				sym = sym.substring(0, sym.length() - 1);   // drop final slash from symbol
				names.push(sym);
				data = null;
				atomic = true;
				
			    }
			
		    
	
		    if (!endsw) {

			    names.push(sym);
			    type = sym;

				if (curFl != null) {

					type = (String) curFl.get(sym);
					if (type == null) {
						network.GenTraceLine("Field " + sym + " not defined for this class");
						return;
					    }
					if (type.charAt(0) == '*')   //test if it's atomic
					    atomic = true;           //  if so, indicate atomic

				    }

				if (!atomic) {

					curFl = (HashMap) classNames.get(type); 
		                                                   // set current field list
	    		    fldLists.push(curFl);                  // push it on stack
	    		    clsNames.push(type);                   // push class name
		  
		            Class c = (Class) classes.get(type);
			        object = c.newInstance();
	                objs.push(object);
	                network.GenTraceLine("Starting class " + type);
					}

		        }
		    else {   // if endsw is on...
			    String tag = (String) names.pop();
			    // GenTraceLine(tag + " popped");
			    if (!sym.equals(tag))  {
				    network.GenTraceLine("Tags don't match");
				    return;
			        }	

	
			    		    
			    if (atomic) {
				    if (data == null) {
				        network.GenTraceLine("Stand-alone tag: " + tag);
				        }
				    else { 
				        String data2 = new String(data);
			            // setAttr(objs.peek(), tag, data2);
			            network.GenTraceLine("Data at " + tag + ": " + data2);
			            data = null;
			            }
				    atomic = false;
			        }
			    
			    else {  // if not atomic

				    network.GenTraceLine("Object popped");
				    object = objs.pop();
				    

				    if (objs.empty()) {
			           Packet op = create(object);
			           network.GenTraceLine("Object output: " + sym);
			           op.drop();
			            /*
		                if (!(outport.send(op))) {
			                op.drop();
				            return;
		                    }
		                */    
				        }
				    else {				
					    // setAttr(objs.peek(), (String) names.peek(), object); 
					    network.GenTraceLine("Field " + sym + " in " + names.peek() +
						    " refers to class " + clsNames.peek());
				        }
				    		    
			        clsNames.pop();
			        fldLists.pop();
			        if (! fldLists.empty())
				        curFl = (HashMap) fldLists.peek();
				    else    
			            curFl = null;
			
			        }
		        }
		    }

		// if we have just read an end tag or a class (non-atomic) name or control, 
		//     skip following blanks!  
		if (endsw || !atomic || control) {
		    while (true) {                           // skip blanks and tabs
	             if (bp.tc(' ', 'o')) continue;
	             if (bp.tc('\t', 'o')) continue;
	             break;
		        }
		    bp.eraseOutput();         // make sure no symbol outstanding
		    control = false;
		    }
	
		// the effect of this is that data will include leading and trailing blanks
		//  or tabs, if any
		 
		while (true) {
		
		   	if (bp.tc('<', 'o')) break;           // look for left bracket
		// here we have found something that is not a <  - it could be
		//  end of file, or it could be data	
		    if (!bp.tu()) {
			   // at this point, we have processed the whole input stream 
			   if (!names.empty()) 
		           network.GenTraceLine("Tags remaining");
		       done = true;      
		       break;
		       } 
		   
	    	}

		if (done) break;
		
		data = bp.getOutStr();
		if (data != null)
		    if (endsw || !atomic)
		        network.GenTraceLine("Characters found not preceded by field start tag : " + data);
		        
		        
		    
		        
		
  }  // end of loop

		if (curFl != null)
		   network.GenTraceLine("Tags not completely processed");
	
	
	    long duration = System.currentTimeMillis() - now;
	    long s = (long) duration / 1000;
	    long ms = (long) duration % 1000;
	    String mss = "000";
	    String ms2 = mss.concat(String.valueOf(ms));
		int i = ms2.length();
		String ms3 = ms2.substring(i - 3, i);	   
	    network.GenTraceLine("Scan time: " + s + '.' + ms3 + " seconds; " + 
		    tagCount + " tags.");
		

}
	protected Object[] introspect() {
		return new Object[] {
		"reads XML strings and generates a stream of Objects",
		"IN", "input", String.class,
			"lines read",
		"OUT", "output", Object.class,
			"objects being created"};
		}
	protected void openPorts() {
		
		inport = openInput("IN");
		inport.setType(String.class);
		
		outport = openOutput("OUT");
		outport.setType(Object.class);
			
		}
/**
 * Insert the method's description here.
 * Creation date: (5/5/00 1:59:01 PM)
 * @param o java.lang.Object
 * @param tag java.lang.String
 * @param data java.lang.String
 */
public void setAttr(Object o, String tag, Object o2) {
	Order order = (Order) o;
	if (tag.equals("webOrderDate"))
		order.webOrderDate = (Date) o2;
	if (tag.equals("expiryDate"))
		order.expiryDate = (Date) o2;		
	}
/**
 * Insert the method's description here.
 * Creation date: (5/5/00 1:59:01 PM)
 * @param o java.lang.Object
 * @param tag java.lang.String
 * @param data java.lang.String
 */
public void setAttr(Object o, String tag, String data) {
	/*  Date date = (Date) o;
	if (tag.equals("D:ccyy"))
		date.ccyy = data;
	if (tag.equals("D:mm"))
		date.mm = data;
	if (tag.equals("D:dd"))
		date.dd = data;	  */
	Order order = (Order) o;
	if (tag.equals("webOrderDate"))
		order.webOrderDate = new Date();
	if (tag.equals("expiryDate"))
		order.expiryDate = new Date();		
	}
}
